##HANDLERS

Un handler est une classe qui va remplacer la logique de base de traitement de données de l'import, pour une table donnée. 

Il s'agit au final d'un service, auquel seront passées les données récupérées par la requête sur la base de données source. 

Il est ensuite possible d'itérer sur chaque ligne du jeu de données et crééer une logique spécifique de traitement et insertion de données. 

Les handlers sont particulièrement utiles quand les données sources sont trop spécifiques par rapport à la structure de la base de données cible, et nécessitent des traitements complexes avant insertion. 

Le handler doit retourner un tableau contenant les potentielles erreurs à faire remonter dans les logs. 

###Créer un handler

La création d'un handler se fait en implémentant l'interface `HandlerInterface`

Exemple :

```php
<?php

declare(strict_types=1);

namespace App\Service\Handler;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManagerInterface;

class CategoriesToTaxonHandler implements HandlerInterface
{
    private Connection $destConnection;
    private EntityManagerInterface $entityManager;

    public function __construct(
        Connection $destConnection,
        EntityManagerInterface $entityManager,
    ) {
        $this->destConnection = $destConnection;
        $this->entityManager = $entityManager;
    }

    public function getType(): string
    {
        return 'app_categorie_to_taxon_handler';
    }

    public function execute(array $data): array
    {
        $errors = [];
        foreach ($data as $line) {
            try {
                // Logique de traitement de données spécifique ici, avec insertion en base de données
                // $line contiendra une ligne du jeu de données configuré dans le mapping
            } catch (\Exception $exception) {
                $errors[] = $exception->getMessage();        
            }
        }
        return $errors;
    }
}
```

La valeur retournée par `getType` sera utilisée pour matcher la configuration `migration_mapping.yaml`. Pour utiliser ce handler on utilisera la config suivante :

```yaml
migration_mapping:
    sql: 'SELECT * FROM categorie'
    ...
    handler: app_categorie_to_taxon_handler
```

L'utilisation d'un handler bypasse complètement les logiques "classique" du migration mapping. De fait, il n'y aura pas de clé "source / dest" insérée dans la table "tmp_dedi_mapping".

Il faudra donc bien penser à enregistrer cette clé manuellement. Pour cette partie, il est possible d'utiliser le service `TemporaryTableWriter` qui enregistre une ligne de mapping dans la table dédiée, sur la base de données cible. 

Exemple de fonction permettant de sauvegarder la clé de mapping : 

```php
class CategorieToTaxonHandler implements HandlerInterface {

    ...
    
    private function saveMapping(int $sourceId, int $destId): void
    {
        $this->destConnection->insert('tmp_dedi_migration_mapping', [
            'source_table_name' => 'categorie',
            'source_column_name' => 'categorie_id',
            'source_id' => $sourceId,
            'dest_table_name' => 'sylius_taxon',
            'dest_column_name' => 'id',
            'dest_id' => $destId,
        ]);
    }
}
```
